import unittest

from tira.check_format import check_format, lines_if_valid

from . import (
    _ERROR,
    _OK,
    EMPTY_OUTPUT,
    JSONL_GZ_OUTPUT_VALID,
    JSONL_OUTPUT_VALID,
    TSV_OUTPUT_VALID,
    TREC_RAG_RESPONSES_VALID
)


class TestCheckJsonlForNonExistingFormats(unittest.TestCase):
    def test_invalid_validator_on_empty_output(self):
        expected = [_ERROR, "No trec-rag-runs were found, "]
        actual = check_format(EMPTY_OUTPUT, "trec-rag-runs")
        self.assertEqual(expected[0], actual[0])
        self.assertIn(expected[1], actual[1])

    def test_valid_jsonl_output_directory(self):
        expected = [_ERROR, "No trec-rag-runs were found, "]
        actual = check_format(JSONL_OUTPUT_VALID, "trec-rag-runs")
        self.assertEqual(expected[0], actual[0])
        self.assertIn(expected[1], actual[1])

    def test_valid_jsonl_output_file(self):
        expected = [_ERROR, "No trec-rag-runs were found, "]
        actual = check_format(JSONL_OUTPUT_VALID / "predictions.jsonl", "trec-rag-runs")
        self.assertEqual(expected[0], actual[0])
        self.assertIn(expected[1], actual[1])

    def test_valid_jsonl_gz_output_directory(self):
        expected = [_ERROR, "No trec-rag-runs were found, "]
        actual = check_format(JSONL_GZ_OUTPUT_VALID, "trec-rag-runs")
        self.assertEqual(expected[0], actual[0])
        self.assertIn(expected[1], actual[1])

    def test_valid_jsonl_gz_output_file(self):
        expected = [_ERROR, "No trec-rag-runs were found, "]
        actual = check_format(JSONL_GZ_OUTPUT_VALID / "predictions.jsonl.gz", "trec-rag-runs")
        self.assertEqual(expected[0], actual[0])
        self.assertIn(expected[1], actual[1])

    def test_invalid_tsv(self):
        expected = [_ERROR, "No trec-rag-runs were found, "]
        actual = check_format(TSV_OUTPUT_VALID, "trec-rag-runs")
        self.assertEqual(expected[0], actual[0])
        self.assertIn(expected[1], actual[1])

    def test_valid_rag_responses(self):
        expected = [_OK, "Valid trec-rag runs."]
        actual = check_format(TREC_RAG_RESPONSES_VALID, "trec-rag-runs")
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])



import os
import unittest
from contextlib import redirect_stderr, redirect_stdout
from io import StringIO
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Optional

from tira.tira_cli import upload_command


def upl(
    dataset: "Optional[str]",
    directory: Path,
    system: str,
    default_task: "Optional[str]" = None,
    tira_vm_id: "Optional[str]" = None,
    anonymous: "Optional[bool]" = False,
):
    os.environ["TIRA_CACHE_DIR"] = str(Path(__file__).parent / "resources" / "local_tira_without_internet")
    stdout = StringIO()
    stderr = StringIO()

    with redirect_stdout(stdout), redirect_stderr(stderr):
        actual = upload_command(dataset, directory, True, system, default_task, tira_vm_id, anonymous)

    del os.environ["TIRA_CACHE_DIR"]

    return actual, stdout.getvalue() + "\n" + stderr.getvalue()


class TestRunUploadDryRun(unittest.TestCase):
    def test_on_non_existing_directory(self):
        ret_code, outp = upl(None, "/this-directory/does-not-exist", None)

        self.assertEqual(1, ret_code)
        self.assertIn("The directory passed via --directory does not exist. Got /this-directory/does-not-exist", outp)

    def test_on_empty_directory_no_dataset(self):
        with TemporaryDirectory() as tmp_dir:
            p = Path(tmp_dir)
            p.mkdir(parents=True, exist_ok=True)
            ret_code, outp = upl(None, str(p), None)

            self.assertEqual(1, ret_code)
            self.assertIn(
                "The dataset is not defined. Please either define a it in a ir-metadata.yml (data: test collection: name: ...) or pass --dataset",
                outp,
            )

    def test_on_empty_directory_with_dataset(self):
        with TemporaryDirectory() as tmp_dir:
            p = Path(tmp_dir)
            p.mkdir(parents=True, exist_ok=True)
            ret_code, outp = upl("dataset-does-not-exist-20241201-training", str(p), None)

            self.assertEqual(1, ret_code)
            self.assertIn(
                "I expected a file ir-metadata.yml in the directory",
                outp,
            )

    def test_on_incomplete_submission(self):
        p = Path(__file__).parent / "resources" / "wows26" / "incomplete"
        ret_code, outp = upl(None, str(p), None)

        self.assertEqual(1, ret_code)
        self.assertIn(
            "is still set to the default value ENTER_VALUE_HERE. Please replace this.",
            outp,
        )

    def test_on_complete_submission_inconsistent_dataset_id(self):
        p = Path(__file__).parent / "resources" / "wows26" / "complete"
        ret_code, outp = upl("dataset-does-not-exist-20241201-training", str(p), None)

        self.assertIn(
            "The dataset for the submission is inconsistent. I got",
            outp,
        )
        self.assertIn(
            "but the metadata of the directory",
            outp,
        )
        self.assertEqual(1, ret_code)

    def test_on_complete_submission_with_inconsistent_team(self):
        p = Path(__file__).parent / "resources" / "wows26" / "complete"
        ret_code, outp = upl(None, str(p), None, tira_vm_id="inconsistent-team")

        self.assertIn(
            "The team for which the submission is to be uploaded is inconsistent. I got inconsistent-team from the --tira-vm-id command line but the metadata of",
            outp,
        )
        self.assertEqual(1, ret_code)

    def test_on_complete_submission_with_consistent_team(self):
        p = Path(__file__).parent / "resources" / "wows26" / "complete"
        ret_code, outp = upl(None, str(p), None, tira_vm_id="some-name")

        self.assertIn(
            "The run is valid. I skip upload to TIRA as --dry-run was passed.",
            outp,
        )
        self.assertEqual(0, ret_code)

    def test_on_complete_submission_withoutdataset_id(self):
        p = Path(__file__).parent / "resources" / "wows26" / "complete"
        ret_code, outp = upl(None, str(p), None)

        self.assertIn(
            "The run is valid. I skip upload to TIRA as --dry-run was passed.",
            outp,
        )
        self.assertEqual(0, ret_code)

    def test_on_complete_submission_consistent_dataset_id(self):
        p = Path(__file__).parent / "resources" / "wows26" / "complete"
        ret_code, outp = upl("some-test-collection", str(p), None)

        self.assertIn(
            "The run is valid. I skip upload to TIRA as --dry-run was passed.",
            outp,
        )
        self.assertEqual(0, ret_code)



import os
import unittest
from pathlib import Path
from tempfile import NamedTemporaryFile, TemporaryDirectory

import pandas as pd

from tira.ir_datasets_loader import get_as_re_rank_input

EXAMPLE_RANKING_01 = """
1 Q0 12 1 10 tag
1 Q0 15 2 9 tag
1 Q0 16 3 6 tag
2 Q0 3 1 1 tag
4 Q0 4 1 10 tag
"""


class TestIrDatasetsLoader(unittest.TestCase):
    def test_loading_of_re_rank_file_depth_10(self):
        expected_path = "-inputs/83051700dcfaf0babc8fa5724dfc5c51/10"
        with TemporaryDirectory() as cache, NamedTemporaryFile() as ranking:
            os.environ["TIRA_CACHE_DIR"] = str(cache)
            ranking = Path(str(ranking.name))
            ranking.write_text(EXAMPLE_RANKING_01)
            actual = get_as_re_rank_input(ranking, 10, "cranfield")

            self.assertIn(str(cache), str(actual))
            self.assertIn(expected_path, str(actual))
            self.assertTrue(str(actual).endswith(expected_path + "/input-data"))
            self.assertTrue((actual / "rerank.jsonl.gz").is_file())
            actual_lines = pd.read_json(actual / "rerank.jsonl.gz", lines=True)
            self.assertEqual(5, len(actual_lines))
            self.assertIn("qid", actual_lines.iloc[0].to_dict())
            self.assertIn("query", actual_lines.iloc[0].to_dict())
            self.assertIn("text", actual_lines.iloc[0].to_dict())

        del os.environ["TIRA_CACHE_DIR"]

    def test_loading_of_re_rank_file_depth_3(self):
        expected_path = "-inputs/83051700dcfaf0babc8fa5724dfc5c51/3"
        with TemporaryDirectory() as cache, NamedTemporaryFile() as ranking:
            os.environ["TIRA_CACHE_DIR"] = str(cache)
            ranking = Path(str(ranking.name))
            ranking.write_text(EXAMPLE_RANKING_01)
            actual = get_as_re_rank_input(ranking, 3, "cranfield")

            self.assertIn(str(cache), str(actual))
            self.assertIn(expected_path, str(actual))
            self.assertTrue(str(actual).endswith(expected_path + "/input-data"))
            self.assertTrue((actual / "rerank.jsonl.gz").is_file())
            actual_lines = pd.read_json(actual / "rerank.jsonl.gz", lines=True)
            self.assertEqual(5, len(actual_lines))
            self.assertIn("qid", actual_lines.iloc[0].to_dict())
            self.assertIn("query", actual_lines.iloc[0].to_dict())
            self.assertIn("text", actual_lines.iloc[0].to_dict())

        del os.environ["TIRA_CACHE_DIR"]

    def test_loading_of_re_rank_file_depth_1(self):
        expected_path = "-inputs/83051700dcfaf0babc8fa5724dfc5c51/1"
        with TemporaryDirectory() as cache, NamedTemporaryFile() as ranking:
            os.environ["TIRA_CACHE_DIR"] = str(cache)
            ranking = Path(str(ranking.name))
            ranking.write_text(EXAMPLE_RANKING_01)
            actual = get_as_re_rank_input(ranking, 1, "cranfield")

            self.assertIn(str(cache), str(actual))
            self.assertIn(expected_path, str(actual))
            self.assertTrue(str(actual).endswith(expected_path + "/input-data"))
            self.assertTrue((actual / "rerank.jsonl.gz").is_file())
            actual_lines = pd.read_json(actual / "rerank.jsonl.gz", lines=True)
            self.assertEqual(3, len(actual_lines))
            self.assertIn("qid", actual_lines.iloc[0].to_dict())
            self.assertIn("query", actual_lines.iloc[0].to_dict())
            self.assertIn("text", actual_lines.iloc[0].to_dict())

        del os.environ["TIRA_CACHE_DIR"]



import os
import unittest

from tira.io_utils import environment_variables_to_forward


class TestForwardOfOpenAiAPIKeys(unittest.TestCase):
    def test_empty_dict_stays_empty_if_no_environment_variable_is_set(self):
        expected = {}

        actual = environment_variables_to_forward()
        self.assertEqual(actual, expected)

    def test_minimal_environment_variables_are_set(self):
        expected = {"OPENAI_API_KEY": "123", "OPENAI_BASE_URL": "456", "OPENAI_MODEL": "789"}

        os.environ["OPENAI_API_KEY"] = "123"
        os.environ["OPENAI_BASE_URL"] = "456"
        os.environ["OPENAI_MODEL"] = "789"

        actual = environment_variables_to_forward(("OPENAI_API_KEY", "OPENAI_BASE_URL", "OPENAI_MODEL"))
        self.assertEqual(actual, expected)

        del os.environ["OPENAI_API_KEY"]
        del os.environ["OPENAI_BASE_URL"]
        del os.environ["OPENAI_MODEL"]

    def test_more_environment_variables_are_set(self):
        expected = {"OPENAI_API_KEY": "123", "OPENAI_BASE_URL": "456", "OPENAI_MODEL": "789", "OPENAI_XYZ": "10"}

        os.environ["OPENAI_API_KEY"] = "123"
        os.environ["OPENAI_BASE_URL"] = "456"
        os.environ["OPENAI_MODEL"] = "789"
        os.environ["OPENAI_XYZ"] = "10"

        actual = environment_variables_to_forward(("OPENAI_API_KEY", "OPENAI_BASE_URL", "OPENAI_MODEL", "OPENAI_XYZ"))
        self.assertEqual(actual, expected)

        del os.environ["OPENAI_API_KEY"]
        del os.environ["OPENAI_BASE_URL"]
        del os.environ["OPENAI_MODEL"]
        del os.environ["OPENAI_XYZ"]

    def test_some_environment_variables_are_ignored(self):
        expected = {"OPENAI_API_KEY": "123", "OPENAI_BASE_URL": "456", "OPENAI_MODEL": "789"}

        os.environ["OPENAI_API_KEY"] = "123"
        os.environ["OPENAI_BASE_URL"] = "456"
        os.environ["OPENAI_MODEL"] = "789"
        os.environ["OPENAI_XYZ"] = "10"

        actual = environment_variables_to_forward(("OPENAI_API_KEY", "OPENAI_BASE_URL", "OPENAI_MODEL"))
        self.assertEqual(actual, expected)

        del os.environ["OPENAI_API_KEY"]
        del os.environ["OPENAI_BASE_URL"]
        del os.environ["OPENAI_MODEL"]
        del os.environ["OPENAI_XYZ"]

    def test_more_environment_variables_are_set_02(self):
        expected = {"OPENAI_API_KEY": "123", "OPENAI_BASE_URL": "456", "OPENAI_MODEL": "789", "OPENAI_XYZ": "11"}

        os.environ["OPENAI_API_KEY"] = "123"
        os.environ["OPENAI_BASE_URL"] = "456"
        os.environ["OPENAI_MODEL"] = "789"
        os.environ["OPENAI_XYZ"] = "11"

        actual = environment_variables_to_forward(("OPENAI_API_KEY", "OPENAI_BASE_URL", "OPENAI_MODEL", "OPENAI_XYZ"))
        self.assertEqual(actual, expected)

        del os.environ["OPENAI_API_KEY"]
        del os.environ["OPENAI_BASE_URL"]
        del os.environ["OPENAI_MODEL"]
        del os.environ["OPENAI_XYZ"]

    def test_failure_thrown_if_model_is_missing(self):
        os.environ["OPENAI_API_KEY"] = "123"
        os.environ["OPENAI_BASE_URL"] = "456"

        with self.assertRaises(ValueError):
            environment_variables_to_forward(("OPENAI_API_KEY", "OPENAI_BASE_URL", "OPENAI_MODEL"))

        del os.environ["OPENAI_API_KEY"]
        del os.environ["OPENAI_BASE_URL"]

    def test_failure_thrown_if_base_url_is_missing(self):
        os.environ["OPENAI_API_KEY"] = "123"
        os.environ["OPENAI_MODEL"] = "789"

        with self.assertRaises(ValueError):
            environment_variables_to_forward(("OPENAI_API_KEY", "OPENAI_BASE_URL", "OPENAI_MODEL"))

        del os.environ["OPENAI_API_KEY"]
        del os.environ["OPENAI_MODEL"]

    def test_failure_thrown_if_api_key_is_missing(self):
        os.environ["OPENAI_BASE_URL"] = "456"
        os.environ["OPENAI_MODEL"] = "789"

        with self.assertRaises(ValueError):
            environment_variables_to_forward(("OPENAI_API_KEY", "OPENAI_BASE_URL", "OPENAI_MODEL"))

        del os.environ["OPENAI_BASE_URL"]
        del os.environ["OPENAI_MODEL"]



import json
import unittest

from tira.check_format import lines_if_valid

from ..format_check import _ERROR, _OK, JSONL_OUTPUT_VALID


def pseudo_watermarking(input_dir, output_dir):
    ret = []
    print("hello world from watermarking")
    for l in lines_if_valid(input_dir, "*.jsonl"):
        l["text"] = "foo"
        ret.append(json.dumps(l))
    (output_dir / "watermarked.jsonl").write_text("\n".join(ret))


def wrong_watermarking(input_dir, output_dir):
    print("A wrong watermarking that does not produce an output")


def pseudo_obfuscation(input_dir, output_dir):
    ret = []
    print("hello world from obfuscation")
    for l in lines_if_valid(input_dir / "01-watermarking", "*.jsonl"):
        l["text"] = "obfuscated"
        l["truth_label"] = "sadasd"
        ret.append(json.dumps(l))
    (output_dir / "obfuscation.jsonl").write_text("\n".join(ret))


def detection_command(input_dir, output_dir):
    ret = []
    print("hello world from detection")
    for l in lines_if_valid(input_dir, "*.jsonl"):
        if "truth_label" in l:
            raise ValueError("dasda")
        if "text" not in l:
            raise ValueError("dsada")

        ret.append(json.dumps(l))
    (output_dir / "predictions.jsonl").write_text("\n".join(ret))


def wrong_detection_command(input_dir, output_dir):
    print("detection that does not write a file...")


COMMAND_TO_MOCK = {
    "wrong-watermarking-command": wrong_watermarking,
    "some-watermarking-command": pseudo_watermarking,
    "some-obfuscation-command": pseudo_obfuscation,
    "some-detection-command": detection_command,
    "wrong-detection-command": wrong_detection_command,
}


def mocked_tira_client():
    class MockedLocalExecution:
        def run(
            self,
            image,
            command,
            input_dir,
            output_dir,
            allow_network,
            additional_volumes,
            cpu_count,
            mem_limit,
            gpu_device_ids,
        ):
            COMMAND_TO_MOCK[command](input_dir, output_dir)

    class MockedTiraClient:
        def __init__(self):
            self.local_execution = MockedLocalExecution()

    return MockedTiraClient()


def run_worklow(workflow_name, workflow_config, software):
    from tira.workflows import run_workflow as rw

    tira_client = mocked_tira_client()
    return rw(JSONL_OUTPUT_VALID, workflow_name, workflow_config, software, None, None, None, None, None, tira_client)


class TestErrorMessageWorkflows(unittest.TestCase):
    def test_for_configuration_with_invalid_watermarking(self):
        expected = [
            _ERROR,
            'Watermarking the text failed. The command "wrong-watermarking-command" did not produce a valid jsonl file.',
        ]
        workflow_name = "pan26-text-watermarking"
        workflow_config = {
            "obfuscation_image": "some-obfuscation-image-01",
            "obfuscation_command": "some-obfuscation-command",
        }
        software = {
            "image": "some-image",
            "watermark_command": "wrong-watermarking-command",
            "detect_command": "some-detection-command",
        }
        actual = run_worklow(workflow_name, workflow_config, software)
        stderr = actual.run / "stderr.txt"
        stdout = actual.run / "stdout.txt"
        self.assertTrue(stderr.exists())
        self.assertTrue(stdout.exists())

        print(stderr.read_text())
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

        stdout_text = stdout.read_text()

        self.assertIn("A wrong watermarking that does not produce an output", stdout_text)
        self.assertIn("Step 1: Watermarking with wrong-watermarking-command", stdout_text)
        self.assertNotIn("hello world from obfuscation", stdout_text)
        self.assertNotIn("hello world from detection", stdout_text)
        self.assertNotIn("Step 5: Detection with some-detection-command", stdout_text)
        self.assertNotIn("Step 3: Obfuscation with some-obfuscation-command", stdout_text)
        self.assertNotIn("The jsonl file has the correct format", stdout_text)

        stderr_text = stderr.read_text()

        self.assertIn("Step 1: Watermarking with wrong-watermarking-command", stderr_text)
        self.assertNotIn("A wrong watermarking that does not produce an output", stderr_text)
        self.assertNotIn("Step 5: Detection with some-detection-command", stderr_text)
        self.assertNotIn("Step 3: Obfuscation with some-obfuscation-command", stderr_text)
        self.assertNotIn("The jsonl file has the correct format", stderr_text)

    def test_for_configuration_with_invalid_detection(self):
        expected = [
            _ERROR,
            'The detection step failed. The command "wrong-detection-command" did not produce a valid jsonl file.',
        ]
        workflow_name = "pan26-text-watermarking"
        workflow_config = {
            "obfuscation_image": "some-obfuscation-image-01",
            "obfuscation_command": "some-obfuscation-command",
        }
        software = {
            "image": "some-image",
            "watermark_command": "some-watermarking-command",
            "detect_command": "wrong-detection-command",
        }
        actual = run_worklow(workflow_name, workflow_config, software)
        stderr = actual.run / "stderr.txt"
        stdout = actual.run / "stdout.txt"
        self.assertTrue(stderr.exists())
        self.assertTrue(stdout.exists())

        print(stderr.read_text())
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

        stdout_text = stdout.read_text()

        self.assertIn("hello world from watermarking", stdout_text)
        self.assertIn("hello world from obfuscation", stdout_text)
        self.assertIn("detection that does not write a file...", stdout_text)
        self.assertIn("Step 5: Detection with wrong-detection-command", stdout_text)
        self.assertIn("Step 3: Obfuscation with some-obfuscation-command", stdout_text)
        self.assertIn("Step 1: Watermarking with some-watermarking-command", stdout_text)
        self.assertIn("The jsonl file has the correct format", stdout_text)

        stderr_text = stderr.read_text()

        self.assertIn("Step 5: Detection with wrong-detection-command", stderr_text)
        self.assertIn("Step 3: Obfuscation with some-obfuscation-command", stderr_text)
        self.assertIn("Step 1: Watermarking with some-watermarking-command", stderr_text)
        self.assertIn("The jsonl file has the correct format", stderr_text)

    def test_for_correct_configuration(self):
        expected = [
            _OK,
            "workflow executed on 'pan26-text-watermarking'.",
        ]
        workflow_name = "pan26-text-watermarking"
        workflow_config = {
            "obfuscation_image": "some-obfuscation-image-01",
            "obfuscation_command": "some-obfuscation-command",
        }
        software = {
            "image": "some-image",
            "watermark_command": "some-watermarking-command",
            "detect_command": "some-detection-command",
        }
        actual = run_worklow(workflow_name, workflow_config, software)
        stderr = actual.run / "stderr.txt"
        stdout = actual.run / "stdout.txt"
        self.assertTrue(stderr.exists())
        self.assertTrue(stdout.exists())

        print(stderr.read_text())
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

        stdout_text = stdout.read_text()

        self.assertIn("hello world from watermarking", stdout_text)
        self.assertIn("hello world from obfuscation", stdout_text)
        self.assertIn("hello world from detection", stdout_text)
        self.assertIn("Step 5: Detection with some-detection-command", stdout_text)
        self.assertIn("Step 3: Obfuscation with some-obfuscation-command", stdout_text)
        self.assertIn("Step 1: Watermarking with some-watermarking-command", stdout_text)
        self.assertIn("The jsonl file has the correct format", stdout_text)

        stderr_text = stderr.read_text()

        self.assertNotIn("hello world from watermarking", stderr_text)
        self.assertNotIn("hello world from obfuscation", stderr_text)
        self.assertNotIn("hello world from detection", stderr_text)
        self.assertIn("Step 5: Detection with some-detection-command", stderr_text)
        self.assertIn("Step 3: Obfuscation with some-obfuscation-command", stderr_text)
        self.assertIn("Step 1: Watermarking with some-watermarking-command", stderr_text)
        self.assertIn("The jsonl file has the correct format", stderr_text)



import unittest

from tira.check_format import check_format

from . import _ERROR, _OK, EMPTY_OUTPUT, JSONL_OUTPUT_VALID, PAN26_TEXT_WATERMARKING


class TestCheckFormatPanTextWatermarking(unittest.TestCase):
    def test_error_message_on_empty_output(self):
        expected = "I expected three directories '01-watermarking', '02-obfuscation', and '03-detection' that each contain jsonl files."
        actual = check_format(EMPTY_OUTPUT, "pan-text-watermarking")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_error_message_on_jsonl_output(self):
        expected = "I expected three directories '01-watermarking', '02-obfuscation', and '03-detection' that each contain jsonl files."
        actual = check_format(JSONL_OUTPUT_VALID, "pan-text-watermarking")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_valid(self):
        expected = "The output is valid for the PAN text watermarking task."
        actual = check_format(PAN26_TEXT_WATERMARKING, "pan-text-watermarking")
        self.assertEqual(_OK, actual[0])
        self.assertIn(expected, actual[1])



import json
from pathlib import Path
from shutil import copyfile, copytree
from typing import Any, Callable, Dict, NamedTuple, Optional

from tira.third_party_integrations import temporary_directory
from tira.tira_client import TiraClient

from .check_format import _fmt, check_format, fmt_message, lines_if_valid


class WorkflowBase:
    def __init__(self):
        self.workflow_configuration = None
        self.software_configuration = None

    def apply_configuration_and_throw_if_invalid(
        self, workflow_configuration: "Optional[Dict[str, Any]]", software: "Optional[Dict[str, Any]]"
    ):
        self.workflow_configuration = workflow_configuration
        self.software_configuration = software

    def run_workflow(
        self,
        system_inputs: Path,
        allow_network: bool,
        additional_volumes,
        cpu_count,
        mem_limit,
        gpu_device_ids,
        tira: "TiraClient",
    ):
        return WorkflowResult(_fmt.OK, "not implemented", None)

    def execute_monitored(self, method: Callable):
        from tira.io_utils import MonitoredExecution

        return MonitoredExecution().run(lambda i: method(i))


class Pan26TextWatermarking(WorkflowBase):
    def apply_configuration_and_throw_if_invalid(
        self, workflow_configuration: "Optional[Dict[str, Any]]", software: "Optional[Dict[str, Any]]"
    ):
        workflow_required = ["obfuscation_image", "obfuscation_command"]
        software_required = ["image", "watermark_command", "detect_command"]

        for k in workflow_required:
            if not workflow_configuration or k not in workflow_configuration:
                raise ValueError(f"The workflow 'pan26-text-watermarking' requires a configuration for '{k}'.")

        for k in software_required:
            if not software or k not in software:
                raise ValueError(f"Software executed for 'pan26-text-watermarking' needs a configuration for '{k}'.")

        super().apply_configuration_and_throw_if_invalid(workflow_configuration, software)

    def run_workflow(
        self,
        system_inputs: Path,
        allow_network: bool,
        additional_volumes,
        cpu_count,
        mem_limit,
        gpu_device_ids,
        tira: "TiraClient",
    ):
        ret = temporary_directory()
        (ret / "output").mkdir(parents=True, exist_ok=True)
        stderr = ret / "stderr.txt"
        stdout = ret / "stdout.txt"

        stderr.write_text(f"# Step 1: Watermarking with {self.software_configuration['watermark_command']}\n\n")
        stdout.write_text(f"# Step 1: Watermarking with {self.software_configuration['watermark_command']}\n\n")

        code, msg = check_format(system_inputs, "*.jsonl")
        if code != _fmt.OK:
            raise ValueError(f"Input is invalid (likely an configuration error): {msg}")

        watermarking_results = self.execute_monitored(
            lambda i: tira.local_execution.run(
                image=self.software_configuration["image"],
                command=self.software_configuration["watermark_command"],
                input_dir=system_inputs,
                output_dir=i,
                allow_network=allow_network,
                additional_volumes=additional_volumes,
                cpu_count=cpu_count,
                mem_limit=mem_limit,
                gpu_device_ids=gpu_device_ids,
            )
        )

        copytree(watermarking_results / "output", ret / "output" / "01-watermarking")
        stderr.write_text(stderr.read_text() + "\n" + (watermarking_results / "stderr.txt").read_text())
        stdout.write_text(stdout.read_text() + "\n" + (watermarking_results / "stdout.txt").read_text())

        code, msg = check_format(ret / "output" / "01-watermarking", "*.jsonl")

        txt = "# Step 2: Check Format\n\t" + fmt_message(msg, code)
        stderr.write_text(stderr.read_text() + "\n\n" + txt)
        stdout.write_text(stdout.read_text() + "\n\n" + txt)

        if code != _fmt.OK:
            return WorkflowResult(
                _fmt.ERROR,
                f"Watermarking the text failed. The command \"{self.software_configuration['watermark_command']}\" did not produce a valid jsonl file.",
                ret,
            )

        obfuscation_inputs = temporary_directory()
        copytree(watermarking_results / "output", obfuscation_inputs / "01-watermarking")
        copytree(system_inputs, obfuscation_inputs / "original")

        obfuscation_results = self.execute_monitored(
            lambda i: tira.local_execution.run(
                image=self.workflow_configuration["obfuscation_image"],
                command=self.workflow_configuration["obfuscation_command"],
                input_dir=obfuscation_inputs,
                output_dir=i,
                allow_network=allow_network,
                additional_volumes=additional_volumes,
                cpu_count=cpu_count,
                mem_limit=mem_limit,
                gpu_device_ids=gpu_device_ids,
            )
        )

        stderr_txt = (
            stderr.read_text()
            + f"\n\n# Step 3: Obfuscation with {self.workflow_configuration['obfuscation_command']}\n\n"
        )
        stdout_txt = (
            stdout.read_text()
            + f"\n\n# Step 3: Obfuscation with {self.workflow_configuration['obfuscation_command']}\n\n"
        )

        stderr_txt += (obfuscation_results / "stderr.txt").read_text()
        stdout_txt += (obfuscation_results / "stdout.txt").read_text()

        copytree(obfuscation_results / "output", ret / "output" / "02-obfuscation")
        code, msg = check_format(ret / "output" / "02-obfuscation", "*.jsonl")

        txt = "\n# Step 4: Check Obfuscation Results\n\t" + fmt_message(msg, code)

        stdout.write_text(stdout_txt + txt)
        stderr.write_text(stderr_txt + txt)

        if code != _fmt.OK:
            return WorkflowResult(_fmt.ERROR, "The step 2 (obfuscation) failed. No valid jsonl file was produced.", ret)

        stderr_txt = (
            stderr.read_text() + f"\n\n# Step 5: Detection with {self.software_configuration['detect_command']}\n\n"
        )
        stdout_txt = (
            stdout.read_text() + f"\n\n# Step 5: Detection with {self.software_configuration['detect_command']}\n\n"
        )

        detection_inputs = temporary_directory()
        detection_lines = []
        for i in lines_if_valid(obfuscation_results / "output", "*.jsonl"):
            del i["truth_label"]
            detection_lines.append(json.dumps(i))

        (detection_inputs / "texts.jsonl").write_text("\n".join(detection_lines))

        detection_results = self.execute_monitored(
            lambda i: tira.local_execution.run(
                image=self.software_configuration["image"],
                command=self.software_configuration["detect_command"],
                input_dir=detection_inputs,
                output_dir=i,
                allow_network=allow_network,
                additional_volumes=additional_volumes,
                cpu_count=cpu_count,
                mem_limit=mem_limit,
                gpu_device_ids=gpu_device_ids,
            )
        )

        stderr_txt += (detection_results / "stderr.txt").read_text()
        stdout_txt += (detection_results / "stdout.txt").read_text()

        copytree(detection_results / "output", ret / "output" / "03-detection")
        code, msg = check_format(ret / "output" / "03-detection", "*.jsonl")

        txt = "\n# Step 6: Check Detection Results\n\t" + fmt_message(msg, code)

        stdout.write_text(stdout_txt + txt)
        stderr.write_text(stderr_txt + txt)

        if code != _fmt.OK:
            return WorkflowResult(
                _fmt.ERROR,
                f"The detection step failed. The command \"{self.software_configuration['detect_command']}\" did not produce a valid jsonl file.",
                ret,
            )

        return WorkflowResult(_fmt.OK, "workflow executed on 'pan26-text-watermarking'.", ret)


WORKFLOWS = {
    "pan26-text-watermarking": Pan26TextWatermarking,
}


class WorkflowResult(NamedTuple):
    level: _fmt
    message: str
    run: Optional[Path]


def run_workflow(
    system_inputs: Path,
    workflow: str,
    workflow_configuration: "Optional[Dict[str, Any]]",
    software: "Optional[Dict[str, Any]]" = None,
    allow_network: bool = False,
    additional_volumes=None,
    cpu_count=1,
    mem_limit=None,
    gpu_device_ids=None,
    tira: "Optional[TiraClient]" = None,
) -> WorkflowResult:
    """Run the specified workflow. Provides debug messages intended for users.

    Args:
        system_inputs (Path): The input to the software.
        workflow (str): The workflow to run.
        workflow_configuration (Optional[Dict[str, Any]]): the configuration of the workflow.
        software (Optional[Dict[str, Any]]): the software to execute in the workflow.
    """
    if workflow not in WORKFLOWS:
        msg = f"No workflow with the identifyier '{workflow}' exists."
        return WorkflowResult(_fmt.ERROR, msg, None)

    workflow_impl = WORKFLOWS[workflow]()
    try:
        workflow_impl.apply_configuration_and_throw_if_invalid(workflow_configuration, software)
    except Exception as e:
        return WorkflowResult(_fmt.ERROR, str(e), None)

    if not tira:
        tira = TiraClient()

    try:
        return workflow_impl.run_workflow(
            system_inputs,
            allow_network,
            additional_volumes,
            cpu_count,
            mem_limit,
            gpu_device_ids,
            tira,
        )
    except Exception as e:
        return WorkflowResult(_fmt.ERROR, str(e), None)



import shutil
import unittest
from pathlib import Path

from tira.rest_api_client import Client
from tira.third_party_integrations import temporary_directory


class TestRestApiNotAvailableTest(unittest.TestCase):

    def clean_tira_cache_dir(self):
        tmp_dir = temporary_directory()
        src_dir = Path(__file__).parent / "resources" / "tira-cache-without-production-system"
        shutil.copy(src_dir / ".tira-settings.json", tmp_dir)

        return tmp_dir

    def test_client_can_be_created(self):
        cache_dir = self.clean_tira_cache_dir()
        tira = Client(tira_cache_dir=Path(cache_dir))
        self.assertIsNotNone(tira)

    def test_all_datasets_works_for_tirex(self):
        cache_dir = self.clean_tira_cache_dir()
        tira = Client(tira_cache_dir=Path(cache_dir))
        actual = tira.datasets("ir-benchmarks")
        self.assertIn("antique-test-20230107-training", actual)

    def test_public_system_details(self):
        cache_dir = self.clean_tira_cache_dir()
        tira = Client(tira_cache_dir=Path(cache_dir))
        actual = tira.public_system_details("tira-ir-starter", "BM25 Re-Rank (tira-ir-starter-pyterrier)")
        self.assertIn("public_image_name", actual)

        self.assertEqual(
            "docker.io/webis/ir-benchmarks-submissions:tira-ir-starter-pyterrier-0-0-1-tira-docker-software-id-plastic-cabin",
            actual["public_image_name"],
        )



import unittest

from ..format_check import _ERROR, _OK, JSONL_OUTPUT_VALID


def mocked_tira_client():
    return None


def run_worklow(workflow_name, workflow_config, software):
    from tira.workflows import run_workflow as rw

    tira_client = mocked_tira_client()
    return rw(JSONL_OUTPUT_VALID, workflow_name, workflow_config, software, None, None, None, None, tira_client)


class TestErrorMessageWorkflows(unittest.TestCase):
    def test_fails_for_non_existing_workflow_01(self):
        expected = [_ERROR, "No workflow with the identifyier 'non-existing-workflow' exists."]
        workflow_name = "non-existing-workflow"
        workflow_config = {}
        software = {}
        actual = run_worklow(workflow_name, workflow_config, software)
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

    def test_fails_for_non_existing_workflow_02(self):
        expected = [_ERROR, "No workflow with the identifyier '1non-existing-workflow1' exists."]
        workflow_name = "1non-existing-workflow1"
        workflow_config = {}
        software = {}
        actual = run_worklow(workflow_name, workflow_config, software)
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

    def test_fails_for_missing_obfuscation_image(self):
        expected = [_ERROR, "The workflow 'pan26-text-watermarking' requires a configuration for 'obfuscation_image'."]
        workflow_name = "pan26-text-watermarking"
        workflow_config = {}
        software = {}
        actual = run_worklow(workflow_name, workflow_config, software)
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

    def test_fails_for_missing_obfuscation_command(self):
        expected = [
            _ERROR,
            "The workflow 'pan26-text-watermarking' requires a configuration for 'obfuscation_command'.",
        ]
        workflow_name = "pan26-text-watermarking"
        workflow_config = {"obfuscation_image": "some-obfuscation-image-01"}
        software = {}
        actual = run_worklow(workflow_name, workflow_config, software)
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

    def test_fails_for_missing_software_image(self):
        expected = [
            _ERROR,
            "Software executed for 'pan26-text-watermarking' needs a configuration for 'image'.",
        ]
        workflow_name = "pan26-text-watermarking"
        workflow_config = {"obfuscation_image": "some-obfuscation-image-01", "obfuscation_command": "some-command"}
        software = {}
        actual = run_worklow(workflow_name, workflow_config, software)
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

    def test_fails_for_missing_watermark_command(self):
        expected = [
            _ERROR,
            "Software executed for 'pan26-text-watermarking' needs a configuration for 'watermark_command'.",
        ]
        workflow_name = "pan26-text-watermarking"
        workflow_config = {"obfuscation_image": "some-obfuscation-image-01", "obfuscation_command": "some-command"}
        software = {"image": "some-image"}
        actual = run_worklow(workflow_name, workflow_config, software)
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])

    def test_fails_for_missing_detect_command(self):
        expected = [
            _ERROR,
            "Software executed for 'pan26-text-watermarking' needs a configuration for 'detect_command'.",
        ]
        workflow_name = "pan26-text-watermarking"
        workflow_config = {"obfuscation_image": "some-obfuscation-image-01", "obfuscation_command": "some-command"}
        software = {"image": "some-image", "watermark_command": "some-command"}
        actual = run_worklow(workflow_name, workflow_config, software)
        self.assertEqual(expected[0], actual[0])
        self.assertEqual(expected[1], actual[1])
